@@ -9,10 +9,12 @@
 #include "yprefs.h"
 #include "ytimer.h"
 #include "udir.h"
+#include "ascii.h"
 #include "intl.h"
 #include "appnames.h"
 
 #define CFGDEF
+#define ICEWMBG
 #include "icewmbg_prefs.h"
 
 char const *ApplicationName = nullptr;
@@ -21,11 +23,12 @@ typedef ref<YImage> Image;
 
 class Cache {
 public:
-    Cache() { }
+    Cache(bool verb) : verbose(verb) { }
     ~Cache() { clear(); }
     void clear() { iname = null; image = null; }
-    Image get(const mstring& name) {
+    Image get(mstring name) {
         if (iname != name) {
+            if (verbose) tlog("load %s", name.c_str());
             iname = name;
             image = YImage::load(iname);
             if (image == null && testOnce(iname, __LINE__)) {
@@ -37,9 +40,10 @@ public:
 private:
     mstring iname;
     Image image;
+    bool verbose;
 };
 
-class Background: public YXApplication, private YTimerListener {
+class Background: public YXApplication {
 public:
     Background(int *argc, char ***argv, bool verbose = false);
     ~Background();
@@ -120,6 +124,7 @@ private:
     bool randInited;
     bool themeInited;
     bool imageInited;
+    bool forceUpdate;
     Strings backgroundImages;
     YColors backgroundColors;
     Strings transparencyImages;
@@ -138,6 +143,7 @@ private:
     lazy<YTimer> cycleTimer;
     lazy<YTimer> checkTimer;
     lazy<YTimer> randrTimer;
+    lazy<YTimer> updateTimer;
 
     Atom _XA_XROOTPMAP_ID;
     Atom _XA_XROOTCOLOR_PIXEL;
@@ -161,6 +167,8 @@ Background::Background(int *argc, char ***argv, bool verb):
     randInited(false),
     themeInited(false),
     imageInited(false),
+    forceUpdate(false),
+    cache(verb),
     mypid(getpid()),
     activeWorkspace(0),
     cycleOffset(0),
@@ -249,7 +257,7 @@ Background::~Background() {
 }
 
 int Background::mainLoop() {
-    update();
+    changeBackground(false);
     if (0 < cycleBackgroundsPeriod) {
         cycleTimer->setTimer(cycleBackgroundsPeriod * 1000L, this, true);
     }
@@ -261,20 +269,36 @@ Atom Background::atom(const char* name) const {
 }
 
 static bool hasImageExt(const char* name) {
-    size_t len = strlen(name);
-    if (len >= 5 && name[len - 4] == '.') {
-        const char* ext = &name[len - 3];
-        if (3 == strspn(ext, "jpngxmJPNGXM"))
-            return true;
-/* #ifdef CONFIG_LIBRSVG
-        if (0 == strcasecmp(ext, "svg"))
-            return true;
-* #endif */
-    }
-    else if (len >= 6 && name[len - 5] == '.') {
-        const char* ext = &name[len - 4];
-        if (0 == strcasecmp(ext, "jpeg"))
-            return true;
+    const char* dot = strrchr(name, '.');
+    if (dot) {
+        const char* ext = dot + 1;
+        size_t len = strlen(ext);
+        if (len > 2 && len < 5) {
+            char low[8];
+            for (int i = 0; i < int(len); ++i)
+                low[i] = ASCII::toLower(ext[i]);
+            low[int(len)] = '\0';
+            if (len == 3) {
+                if (strcmp(low, "xpm") == 0 ||
+                    strcmp(low, "png") == 0 ||
+                    strcmp(low, "jpg") == 0 ||
+                    (strcmp(low, "jxl") == 0 && YImage::supportsFormat(low)) ||
+                    (strcmp(low, "jp2") == 0 && YImage::supportsFormat(low)) ||
+                    (strcmp(low, "raw") == 0 && YImage::supportsFormat(low)) ||
+                    (strcmp(low, "svg") == 0 && YImage::supportsFormat(low)) ||
+                    (strcmp(low, "tga") == 0 && YImage::supportsFormat(low)) ||
+                    (strcmp(low, "tif") == 0 && YImage::supportsFormat(low)))
+                    return true;
+            }
+            if (len == 4) {
+                if (strcmp(low, "jpeg") == 0 ||
+                    (strcmp(low, "avif") == 0 && YImage::supportsFormat(low)) ||
+                    (strcmp(low, "svgz") == 0 && YImage::supportsFormat(low)) ||
+                    (strcmp(low, "tiff") == 0 && YImage::supportsFormat(low)) ||
+                    (strcmp(low, "webp") == 0 && YImage::supportsFormat(low)))
+                    return true;
+            }
+        }
     }
     return false;
 }
@@ -428,6 +452,14 @@ bool Background::handleTimer(YTimer* timer) {
             update(true);
         }
     }
+    else if (timer == updateTimer) {
+        if (cycleTimer && cycleTimer->expires()) {
+            cycleTimer->startTimer();
+            cycleOffset += desktopCount;
+        }
+        changeBackground(forceUpdate);
+        forceUpdate = false;
+    }
     return false;
 }
 
@@ -481,15 +513,21 @@ YColor Background::getTransparencyColor() {
 }
 
 void Background::randinit(long seed) {
+#ifndef HAVE_ARC4RANDOM_UNIFORM
     if (randInited == false) {
         timeval now = walltime();
         srand(unsigned((seed + mypid) * now.tv_sec + now.tv_usec));
         randInited = true;
     }
+#endif
 }
 
 int Background::randmax(int upper) {
+#ifdef HAVE_ARC4RANDOM_UNIFORM
+    return arc4random_uniform(upper + 1);
+#else
     return rand() / (RAND_MAX / (upper + 1) + 1);
+#endif
 }
 
 void Background::reshuffle() {
@@ -524,9 +562,8 @@ void Background::startShuffle() {
 }
 
 void Background::update(bool force) {
-    activeWorkspace = getWorkspace();
-    if (verbose) tlog("update %s %d", boolstr(force), activeWorkspace);
-    changeBackground(force);
+    forceUpdate |= force;
+    updateTimer->setTimer(0L, this, true);
 }
 
 long* Background::getLongProperties(Atom property, int n) const {
@@ -662,6 +699,15 @@ ref<YPixmap> Background::renderBackground(Image back, YColor color) {
         }
         ref<YPixmap> pixm = todraw->renderToPixmap(g.rdepth());
         if (pixm != null) {
+            if (verbose) {
+                tlog(" ... @%d,%d %ux%u+%d+%d",
+                             max(0, int(bw - width) / 2),
+                             max(0, int(bh - height) / 2),
+                             min(width, bw),
+                             min(height, bh),
+                             max(x, x + int(width - bw) / 2),
+                             max(y, y + int(height - bh) / 2));
+            }
             g.drawPixmap(pixm,
                          max(0, int(bw - width) / 2),
                          max(0, int(bh - height) / 2),
@@ -676,6 +722,10 @@ ref<YPixmap> Background::renderBackground(Image back, YColor color) {
 }
 
 void Background::changeBackground(bool force) {
+    activeWorkspace = getWorkspace();
+    if (verbose)
+        tlog("update f=%s ws=%d", boolstr(force), activeWorkspace);
+
     Image backgroundImage = getBackgroundImage();
     YColor backgroundColor(getBackgroundColor());
     if (force == false) {
@@ -896,7 +946,8 @@ bool Background::become(bool replace) {
 }
 
 void Background::printOptions() {
-    for (cfoption* o = icewmbg_prefs; o && o->type; ++o) {
+    for (int k = 0; k < 10 && icewmbg_prefs[k].type; ++k) {
+        cfoption* o = &icewmbg_prefs[k];
         switch (o->type) {
             case cfoption::CF_BOOL:
                 printf("%-26s = %d\n", o->name, *o->v.b.bool_value);
@@ -970,6 +1021,8 @@ static const char* get_help_text() {
     " center:1 scaled:1 = fill one dimension and keep aspect ratio\n"
     " center:0 scaled:1 = fill both dimensions and keep aspect ratio\n"
     "\n");
+    /* avoid work for translators for now:
+    "  --postpreferences    Print preferences after all processing.\n" */
 }
 
 static void print_help_xit() {
@@ -1001,7 +1054,20 @@ static void bgLoadConfig(const char *configFile, const char *overrideTheme)
         };
         YConfig(theme_prefs).load(configFile).load("theme");
     }
-    YConfig(icewmbg_prefs).load(configFile).loadTheme().loadOverride();
+    YConfig conf(icewmbg_prefs);
+    conf.load(configFile);
+
+    cfoption* o = icewmbg_prefs;
+    while (o->type && *o->name != 'X')
+        ++o;
+    cfoption::OptionType type = cfoption::CF_NONE;
+    if (o)
+        swap(type, o->type);
+    conf.loadTheme();
+    if (o)
+        swap(type, o->type);
+
+    conf.loadOverride();
 }
 
 static void bgParse(const char* name, const char* value) {
@@ -1192,13 +1258,13 @@ int main(int argc, char **argv) {
                 warn(_("Unrecognized option '%s'."), *arg);
         }
         else if (hasImageExt(*arg)) {
-            image += *arg;
+            appendFile(image, *arg);
         }
         else {
             upath path(*arg + (**arg == '!'));
             path = path.expand();
             if (path.dirExists() && path.isSearchable())
-                image += *arg;
+                appendFile(image, *arg);
         }
     }
 
