@@ -16,6 +16,10 @@ enum WMAction {
     ICEWM_ACTION_SUSPEND = 9,
     ICEWM_ACTION_WINOPTIONS = 10,
     ICEWM_ACTION_RELOADKEYS = 11,
+    ICEWM_ACTION_ICEWMBG = 12,
+    ICEWM_ACTION_REFRESH = 13,
+    ICEWM_ACTION_HIBERNATE = 14,
+    ICEWM_ACTION_TOOLBAR = 15,
 };
 
 enum RebootShutdown {
@@ -119,10 +123,26 @@ enum EAction {
     actionUntab              = 255,
     actionRename             = 257,
     actionSysDialog          = 259,
+    actionIcewmbg            = 261,
+    actionHibernate          = 263,
+    actionToolbar            = 265,
+
+    actionHelpManual         = 267,
+    actionHelpIcewm          = 269,
+    actionHelpIcewmbg        = 271,
+    actionHelpIcesound       = 273,
+    actionHelpIcesh          = 275,
+    actionHelpEnv            = 277,
+    actionHelpKeys           = 279,
+    actionHelpStartup        = 281,
+    actionHelpToolbar        = 283,
+    actionHelpWinoptions     = 285,
+
 };
 
 bool canShutdown(RebootShutdown reboot);
 bool canSuspend();
+bool canHibernate();
 bool canLock();
 /**
  * Basic check whether a shell command could possibly be run.
